#ifndef CHANNELMAPSERVICE__H
#define CHANNELMAPSERVICE__H
////////////////////////////////////////////////////////////////////////
/// \file ChannelMap_service.cc
///
///
/// \author Andrey Sheshukov
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Services/Registry/ActivityRegistry.h"
#include "art/Framework/Services/Registry/ServiceMacros.h"
#include "messagefacility/MessageLogger/MessageLogger.h"

#include "OnlineCalibration/HitMap.h"

#include "OnlineCalibration/FileWatcher.h"

namespace novaddt{
    class StaticMapService;
}

//A service for access to static noise hit map object

class novaddt::StaticMapService{
    public:
        StaticMapService(fhicl::ParameterSet const& pset, art::ActivityRegistry& reg);
        //get map for reading
        const novaddt::calib::FDHitMap& getMap() const{return fMap;}
        //map IO from/to file
        void reload(){readMapFromFile(fMapSource);}
        void readMapFromFile(const std::string& fname);
    protected:
        void preProcessEvent(const art::Event& event);
    private:
        std::string fMapSource; //< filename to read map for each run
        FileWatcher fFileWatcher;
        calib::FDHitMap fMap;
};

DECLARE_ART_SERVICE(novaddt::StaticMapService,LEGACY);
#endif
