////////////////////////////////////////////////////////////////////////
// Class:       ChannelMapMaker
// Module Type: analyzer
// File:        ChannelMapMaker_module.cc
//
// Generated at Fri Feb 24 17:38:07 2017 by Andrey Sheshukov using artmod
// from cetpkgsupport v1_10_02.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"
#include "DDTBaseDataProducts/HitList.h"
#include "OnlineCalibration/HitMap.h"
#include "OnlineCalibration/FileWatcher.h"
#include <sstream>

namespace novaddt {
  class ChannelMapMaker;
}

class novaddt::ChannelMapMaker : public art::EDAnalyzer{
public:
  explicit ChannelMapMaker(fhicl::ParameterSet const & p);
  // The destructor generated by the compiler is fine for classes
  // without bare pointers or other resource use.

  // Plugins should not be copied or assigned.
  ChannelMapMaker(ChannelMapMaker const &) = delete;
  ChannelMapMaker(ChannelMapMaker &&) = delete;
  ChannelMapMaker & operator = (ChannelMapMaker const &) = delete;
  ChannelMapMaker & operator = (ChannelMapMaker &&) = delete;

  // Required functions.
  void analyze(art::Event const & e) override;

  void saveMap();
  void saveMapTo(std::string filename);
  void reset();

private:

  // Declare member data here.
    novaddt::calib::FDHitMap fHitMap;
    //configuration parameters
    art::InputTag fHitsTag; //where to read the hits for noise activity
    std::string fOutputFile; //where to write map
    std::string fWatchFile; //where to write map
    uint32_t fEventsProcessed=0;
    FileWatcher fFileWatcher;
};


novaddt::ChannelMapMaker::ChannelMapMaker(fhicl::ParameterSet const & p):
  EDAnalyzer(p),
  fHitsTag(p.get<std::string>("hits_tag")),
  fOutputFile(p.get<std::string>("output")),
  fWatchFile (p.get<std::string>("watch")),
  fFileWatcher(fWatchFile)
{
    mf::SetModuleName("ChannelMapMaker");
    mf::LogInfo("Configuration")<<"input_hits:\""<<fHitsTag
        <<"\nOutput map to: "<<fOutputFile
        <<"\nWatch file: "<<fWatchFile
        <<std::endl;
}

void novaddt::ChannelMapMaker::analyze(art::Event const & e)
{
    //get the hits which we'll use to fill the map
    auto hits=e.getValidHandle<novaddt::HitList>(fHitsTag);
    //fill the map
    for(const auto &hit: *hits)
        fHitMap[hit]+=1;
    //Increment events counter
    ++fEventsProcessed;

    //check if we should do the rollover
    if(fFileWatcher.isModified()){
        mf::LogInfo("MapRollover")<<" Time to save the map file"<<std::endl;
        saveMap();
        reset();
        fFileWatcher.update();
    }
}

void novaddt::ChannelMapMaker::saveMap(){
    //normalize the map to 1 event
    fHitMap.Scale(1./fEventsProcessed);
    saveMapTo(fOutputFile);
}
  
void novaddt::ChannelMapMaker::reset(){
    fHitMap.reset(); //zero the map befor filling 
    fEventsProcessed=0; //zero the events counter
}



void novaddt::ChannelMapMaker::saveMapTo(std::string filename){
    mf::LogInfo("Progress")<<"Saving NoiseMap with "<<fEventsProcessed<<" events to file:"<<filename <<std::endl;
    //save map to the file
    std::ofstream s(filename.data(),std::ofstream::binary);
    s<<fHitMap;
}

DEFINE_ART_MODULE(novaddt::ChannelMapMaker)
