////////////////////////////////////////////////////////////////////////
// Class:       NoiseMapAna
// Module Type: analyzer
// File:        NoiseMapAna_module.cc
//
// Generated at Tue Aug  2 14:48:49 2016 by Andrew S. Hirsch using artmod
// from cetpkgsupport v1_10_02.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"

#include "OnlineCalibration/StaticMapService.h"
#include "art/Framework/Services/Optional/TFileService.h"
#include <TH2I.h>

namespace novaddt {
  class NoiseMapAna;
}


void MakeHist(int n)
{
    ///use the shared noise map manager
    art::ServiceHandle<novaddt::StaticMapService> cms;
   //lock before reading
    auto &NoiseMap = cms->getMap();

    //make noise histogram
    art::ServiceHandle<art::TFileService> tfs;

    auto h1=tfs->make<TH1D>(Form("h1d_%d",n),
                        Form("Noise, event #%d",n),
                        1000,0,200);
    auto h2=tfs->make<TH2D>(Form("h2d_%d",n),
                        Form("Noise Map, event #%d",n),
                        NoiseMap.Nplanes(),0,NoiseMap.Nplanes(),
                        NoiseMap.Ncells(),0,NoiseMap.Ncells());
    uint64_t sum=0;
    double mean=0, rms=0;
    for(size_t nc=0;nc<NoiseMap.Ncells();++nc){
        for(size_t np=0;np<NoiseMap.Nplanes();++np){
            auto val=NoiseMap(novaddt::Plane(np),novaddt::Cell(nc));
            sum+=val;
            rms+=val*val;
            h1->Fill(val);
            h2->SetBinContent(np,nc,val);
         }
    }
    mean=sum;
    mean=mean/NoiseMap.Length();
    rms=rms/NoiseMap.Length()-mean*mean;
    
    mf::LogInfo("NoiseMapAna")<<" activity["<<sum<<"] = "<<mean<<" +- "<<sqrt(rms);
}

class novaddt::NoiseMapAna : public art::EDAnalyzer {
public:
  explicit NoiseMapAna(fhicl::ParameterSet const & p);
  // The destructor generated by the compiler is fine for classes
  // without bare pointers or other resource use.

  // Plugins should not be copied or assigned.
  NoiseMapAna(NoiseMapAna const &) = delete;
  NoiseMapAna(NoiseMapAna &&) = delete;
  NoiseMapAna & operator = (NoiseMapAna const &) = delete;
  NoiseMapAna & operator = (NoiseMapAna &&) = delete;

  // Required functions.
  void analyze(art::Event const & e) override;

  void endRun(art::Run const & r) override{
      art::ServiceHandle<novaddt::StaticMapService> cms;
      MakeHist(r.run());
  }

private:


};




novaddt::NoiseMapAna::NoiseMapAna(fhicl::ParameterSet const & p)
  :
  EDAnalyzer(p)  // ,
 // More initializers here.
{}

void novaddt::NoiseMapAna::analyze(art::Event const & e)
{
  // Implementation of required member function here.
}

DEFINE_ART_MODULE(novaddt::NoiseMapAna)
