#ifndef CHANNELMAP_SERVICE__CC
#define CHANNELMAP_SERVICE__CC
////////////////////////////////////////////////////////////////////////
/// \file ChannelMap_service.cc
///  
/// \brief A service to manipulate channel map, containing activity
///
/// \author Andrey Sheshukov
////////////////////////////////////////////////////////////////////////

#include "messagefacility/MessageLogger/MessageLogger.h"
#include "OnlineCalibration/StaticMapService.h"
#include <boost/filesystem.hpp>


namespace novaddt{

StaticMapService::StaticMapService
    (fhicl::ParameterSet const& p, art::ActivityRegistry& reg):
    fMapSource(p.get<std::string>("source")),
    fFileWatcher(fMapSource)
{
    ::mf::SetModuleName("StaticMapService");
    ::mf::LogInfo("Configuration")<<"map source:"<<fMapSource;
    
    //set watcher to monitor file changes in the MapFile
    reg.sPreProcessEvent.watch(&novaddt::StaticMapService::preProcessEvent, *this);
    
}

void StaticMapService::preProcessEvent(const art::Event& event){
    //reload themap from a new file, when the run number was changed
    if(fFileWatcher.isModified())
    {
        mf::LogInfo("MapRollover")<<" Map file changed. Updating the map..."<<std::endl;
        fFileWatcher.update();
        reload();
    }
}


void StaticMapService::readMapFromFile(const std::string& fname)
{
        ::mf::LogInfo("Progress")<<"reading map from file \""<<fname<<"\"";
        //block until file is created
        if(!boost::filesystem::exists(fname)){
            ::mf::LogWarning("Progress")<<"File "<<fname<<"does not exist.\n";
            return;
        }

        std::ifstream s(fname.data(),std::ifstream::binary);
        if(!s.good()){
            ::mf::LogError("Progress")<<" error opening file \""<<fname<<"\"";
            return;
        }
        s>>fMap;
        ::mf::LogInfo("Progress")<<"Read map successfully"<<std::endl;
}

DEFINE_ART_SERVICE(StaticMapService);
}
#endif
